@@ -185,7 +185,7 @@ class EnamlImporter(AbstractEnamlImporter):
     This importer adopts the Python 3 conventions and scheme for creating
     the cached files and setting the __file__ attribute on the module.
     See this discussion thread for more info:
-        http://www.mail-archive.com/python-dev@python.org/msg45203.html
+    http://www.mail-archive.com/python-dev@python.org/msg45203.html
 
     """
     @classmethod
@@ -342,7 +342,7 @@ class EnamlImporter(AbstractEnamlImporter):
                 return (code, file_info.src_path)
 
         # Otherwise, compile from source and attempt to cache
-        with open(file_info.src_path) as src_file:
+        with open(file_info.src_path, 'rU') as src_file:
             src = src_file.read()
         ast = parse(src)
         code = EnamlCompiler.compile(ast, file_info.src_path)
