@@ -12,7 +12,7 @@ from .lexer import syntax_error, EnamlLexer, ParsingError
 
 
 #------------------------------------------------------------------------------
-# Parser Setup 
+# Parser Setup
 #------------------------------------------------------------------------------
 # Lexer tokens which need to be exposed to the parser
 tokens = EnamlLexer.tokens
@@ -92,7 +92,7 @@ except AttributeError:
         return call
 
 
-# The disallowed ast node types for ast.Store contexts and 
+# The disallowed ast node types for ast.Store contexts and
 # the associated message tag for error reporting.
 context_disallowed = {
     ast.Lambda: 'lambda',
@@ -150,7 +150,7 @@ augassign_table = {
 
 
 class FakeToken(object):
-    """ A fake token used to store the lexer before calling the 
+    """ A fake token used to store the lexer before calling the
     syntax error functions.
 
     """
@@ -161,7 +161,7 @@ class FakeToken(object):
 
 def translate_operator(op):
     """ Converts a symbolic operator into a string of the form
-    __operator_<name>__ where <name> is result of translating the 
+    __operator_<name>__ where <name> is result of translating the
     symbolic operator using the operator_table.
 
     """
@@ -197,13 +197,13 @@ def set_context(node, ctx, p):
     else:
         msg = 'unexpected expression in assignment %d (line %d)'
         raise SystemError(msg % (node_type.__name__, node.lineno))
-    
+
     if err_msg:
         m = 'assign to' if ctx == Store else 'delete'
         msg = "can't %s %s" % (m, err_msg)
         tok = FakeToken(p.lexer.lexer, p.lexer.lexer.lineno - 1)
         syntax_error(msg, tok)
-            
+
     if items is not None:
         for item in items:
             set_context(item, ctx, p)
@@ -236,7 +236,35 @@ def ast_for_dotted_name(dotted_name):
     return res
 
 
-def build_attr_declaration(kw, name, type_node, default, lineno, p):
+# The nodes which can be inverted to form an assignable expression.
+_INVERTABLE = (ast.Name, ast.Attribute, ast.Call, ast.Subscript)
+
+def validate_invertable(node, lineno, p):
+    """ Validates that its possible for the compiler to generated
+    inversion code for the given ast node.
+
+    Currently, code is invertable if the expression terminates with
+    a node of the following types: Name, Attribute, Call, Subscript.
+
+    Parameters
+    ----------
+    node : ast.AST
+        The ast expression node to validate.
+
+    lineno : int
+        The line number of the declaration.
+
+    p : Yacc Production
+        The Ply object passed to the parser rule. This is used to
+        extract the filename for syntax error reporting.
+
+    """
+    if not isinstance(node, _INVERTABLE):
+        msg = "can't assign to expression of this form"
+        syntax_error(msg, FakeToken(p.lexer.lexer, lineno))
+
+
+def build_attr_declaration(kw, name, attr_type, default, lineno, p):
     """ Builds an ast node for an attr or event declaration.
 
     Parameters
@@ -244,24 +272,23 @@ def build_attr_declaration(kw, name, type_node, default, lineno, p):
     kw : string
         The keyword used in the declaration. A syntax error is raised if
         this is not 'attr' or 'event'.
-    
+
     name : string
         The name of the attribute or event being declared.
-    
-    type_node : enaml_ast.Python node or None
-        The expression node for the type being declared, or None if not 
-        using a type.
-    
+
+    attr_type : str or None
+        The type being declared, or None if not using a type.
+
     default : AttributeBinding or None
         The default attribute binding or None if not supply the default.
 
     lineno : int
         The line number of the declaration.
-    
+
     p : Yacc Production
-        The Ply object passed to the parser rule. This is used to 
+        The Ply object passed to the parser rule. This is used to
         extract the filename for syntax error reporting.
-    
+
     Returns
     -------
     result : AttributeDeclaration
@@ -273,11 +300,11 @@ def build_attr_declaration(kw, name, type_node, default, lineno, p):
         syntax_error(msg, FakeToken(p.lexer.lexer, lineno))
     if kw == 'attr':
         res = enaml_ast.AttributeDeclaration(
-            name, type_node, default, False, lineno,
+            name, attr_type, default, False, lineno,
         )
     else:
         res = enaml_ast.AttributeDeclaration(
-            name, type_node, default, True, lineno,
+            name, attr_type, default, True, lineno,
         )
     return res
 
@@ -334,7 +361,7 @@ def p_enaml2(p):
 
 def p_enaml_module(p):
     ''' enaml_module : enaml_module_body '''
-    # Separate the Python statements from the declarations and 
+    # Separate the Python statements from the declarations and
     # collect them into their node
     python_nodes = []
     body_nodes = []
@@ -376,16 +403,30 @@ def p_enaml_module_item1(p):
 #------------------------------------------------------------------------------
 # Declaration
 #------------------------------------------------------------------------------
-def p_declaration(p):
+def p_declaration1(p):
     ''' declaration : ENAMLDEF NAME LPAR NAME RPAR COLON declaration_body '''
-    lineno = p.lineno(1)
-    name = p[2]
     doc, idn, items = p[7]
-    base = ast.Expression(body=ast.Name(id=p[4], ctx=Load))
-    base.lineno = lineno
-    ast.fix_missing_locations(base)
-    base_node = enaml_ast.Python(base, lineno)
-    p[0] = enaml_ast.Declaration(name, base_node, idn, doc, items, lineno)
+    p[0] = enaml_ast.Declaration(p[2], p[4], idn, doc, items, p.lineno(1))
+
+
+def p_declaration2(p):
+    ''' declaration : ENAMLDEF NAME LPAR NAME RPAR COLON PASS NEWLINE '''
+    p[0] = enaml_ast.Declaration(p[2], p[4], None, '', [], p.lineno(1))
+
+
+def p_declaration3(p):
+    ''' declaration : ENAMLDEF NAME LPAR NAME RPAR COLON NAME COLON declaration_body '''
+    lineno = p.lineno(1)
+    doc, idn, items = p[9]
+    if idn is not None:
+        msg = 'multiple identifiers declared'
+        syntax_error(msg, FakeToken(p.lexer.lexer, lineno))
+    p[0] = enaml_ast.Declaration(p[2], p[4], p[7], doc, items, lineno)
+
+
+def p_declaration4(p):
+    ''' declaration : ENAMLDEF NAME LPAR NAME RPAR COLON NAME COLON PASS NEWLINE '''
+    p[0] = enaml_ast.Declaration(p[2], p[4], p[7], '', [], p.lineno(1))
 
 
 def p_declaration_body1(p):
@@ -466,12 +507,7 @@ def p_attribute_declaration1(p):
 
 def p_attribute_declaration2(p):
     ''' attribute_declaration : NAME NAME COLON NAME NEWLINE '''
-    lineno = p.lineno(1) 
-    expr = ast.Expression(body=ast.Name(id=p[4], ctx=Load))
-    expr.lineno = lineno
-    ast.fix_missing_locations(expr)
-    expr_node = enaml_ast.Python(expr, lineno)
-    p[0] = build_attr_declaration(p[1], p[2], expr_node, None, p.lineno(1), p)
+    p[0] = build_attr_declaration(p[1], p[2], p[4], None, p.lineno(1), p)
 
 
 def p_attribute_declaration3(p):
@@ -485,13 +521,9 @@ def p_attribute_declaration3(p):
 def p_attribute_declaration4(p):
     ''' attribute_declaration : NAME NAME COLON NAME binding '''
     lineno = p.lineno(1)
-    expr = ast.Expression(body=ast.Name(id=p[4], ctx=Load))
-    expr.lineno = lineno
-    ast.fix_missing_locations(expr)
-    expr_node = enaml_ast.Python(expr, lineno)
     name = p[2]
     binding = enaml_ast.AttributeBinding(name, p[5], lineno)
-    p[0] = build_attr_declaration(p[1], name, expr_node, binding, lineno, p)
+    p[0] = build_attr_declaration(p[1], name, p[4], binding, lineno, p)
 
 
 #------------------------------------------------------------------------------
@@ -509,12 +541,41 @@ def p_identifier(p):
 #------------------------------------------------------------------------------
 # Instantiation
 #------------------------------------------------------------------------------
-def p_instantiation(p):
+def p_instantiation1(p):
     ''' instantiation : NAME COLON instantiation_body '''
     identifier, items = p[3]
     p[0] = enaml_ast.Instantiation(p[1], identifier, items, p.lineno(1))
 
 
+def p_instantiation2(p):
+    ''' instantiation : NAME COLON attribute_binding '''
+    p[0] = enaml_ast.Instantiation(p[1], None, [p[3]], p.lineno(1))
+
+
+def p_instantiation3(p):
+    ''' instantiation : NAME COLON PASS NEWLINE '''
+    p[0] = enaml_ast.Instantiation(p[1], None, [], p.lineno(1))
+
+
+def p_instantiation4(p):
+    ''' instantiation : NAME COLON NAME COLON instantiation_body '''
+    identifier, items = p[5]
+    if identifier is not None:
+        msg = 'multiple identifiers declared'
+        syntax_error(msg, FakeToken(p.lexer.lexer, p.lineno(1)))
+    p[0] = enaml_ast.Instantiation(p[1], p[3], items, p.lineno(1))
+
+
+def p_instantiation5(p):
+    ''' instantiation : NAME COLON NAME COLON attribute_binding '''
+    p[0] = enaml_ast.Instantiation(p[1], p[3], [p[5]], p.lineno(1))
+
+
+def p_instantiation6(p):
+    ''' instantiation : NAME COLON NAME COLON PASS NEWLINE '''
+    p[0] = enaml_ast.Instantiation(p[1], p[3], [], p.lineno(1))
+
+
 def p_instantiation_body1(p):
     ''' instantiation_body : NEWLINE INDENT instantiation_body_items DEDENT '''
     # Filter out any pass statements
@@ -569,10 +630,8 @@ def p_attribute_binding(p):
 
 def p_binding1(p):
     ''' binding : EQUAL test NEWLINE
-                | COLONEQUAL test NEWLINE
-                | LEFTSHIFT test NEWLINE
-                | RIGHTSHIFT test NEWLINE '''
-    lineno = p.lineno(1) 
+                | LEFTSHIFT test NEWLINE '''
+    lineno = p.lineno(1)
     operator = translate_operator(p[1])
     expr = ast.Expression(body=p[2])
     expr.lineno = lineno
@@ -582,6 +641,19 @@ def p_binding1(p):
 
 
 def p_binding2(p):
+    ''' binding : COLONEQUAL test NEWLINE
+                | RIGHTSHIFT test NEWLINE '''
+    lineno = p.lineno(1)
+    validate_invertable(p[2], lineno, p)
+    operator = translate_operator(p[1])
+    expr = ast.Expression(body=p[2])
+    expr.lineno = lineno
+    ast.fix_missing_locations(expr)
+    expr_node = enaml_ast.Python(expr, lineno)
+    p[0] = enaml_ast.BoundExpression(operator, expr_node, lineno)
+
+
+def p_binding3(p):
     ''' binding : DOUBLECOLON suite '''
     lineno = p.lineno(1)
     operator = translate_operator(p[1])
@@ -618,7 +690,14 @@ def p_suite2(p):
 
 def p_stmt_list1(p):
     ''' stmt_list : stmt stmt_list '''
-    p[0] = [p[1]] + p[2]
+    # stmt may be a list of simple_stmt due to this piece of grammar:
+    # simple_stmt: small_stmt (';' small_stmt)* [';'] NEWLINE
+    stmt = p[1]
+    if isinstance(stmt, list):
+        res = stmt + p[2]
+    else:
+        res = [stmt] + p[2]
+    p[0] = res
 
 
 def p_stmt_list2(p):
@@ -688,7 +767,7 @@ def p_small_stmt1(p):
                    | del_stmt
                    | pass_stmt
                    | flow_stmt
-                   | import_stmt 
+                   | import_stmt
                    | global_stmt
                    | exec_stmt
                    | assert_stmt '''
@@ -804,7 +883,7 @@ def p_pass_stmt(p):
 
 def p_flow_stmt(p):
     ''' flow_stmt : break_stmt
-                  | continue_stmt 
+                  | continue_stmt
                   | return_stmt
                   | raise_stmt
                   | yield_stmt '''
@@ -988,7 +1067,7 @@ def p_expr_stmt2(p):
 def p_expr_stmt3(p):
     ''' expr_stmt : testlist equal_list '''
     all_items = [p[1]] + p[2]
-    targets = map(ast_for_testlist, all_items) 
+    targets = map(ast_for_testlist, all_items)
     value = targets.pop()
     for item in targets:
         if type(item) == ast.Yield:
@@ -997,7 +1076,7 @@ def p_expr_stmt3(p):
         set_context(item, Store, p)
     assg = ast.Assign()
     assg.targets = targets
-    assg.value = value   
+    assg.value = value
     p[0] = assg
 
 
@@ -1062,12 +1141,12 @@ def p_testlist_list2(p):
 
 
 def p_compound_stmt(p):
-    ''' compound_stmt : if_stmt 
+    ''' compound_stmt : if_stmt
                       | while_stmt
                       | for_stmt
                       | try_stmt
                       | with_stmt
-                      | funcdef 
+                      | funcdef
                       | classdef
                       | decorated '''
     p[0] = p[1]
@@ -1144,7 +1223,7 @@ def p_elif_stmt(p):
     ast.fix_missing_locations(if_stmt)
     p[0] = if_stmt
 
- 
+
 def p_else_stmt(p):
     ''' else_stmt : ELSE COLON suite '''
     p[0] = p[3]
@@ -1299,7 +1378,7 @@ def p_except_clause2(p):
 
 
 def p_except_clause3(p):
-    ''' except_clause : EXCEPT test AS test COLON suite 
+    ''' except_clause : EXCEPT test AS test COLON suite
                       | EXCEPT test COMMA test COLON suite '''
     excpt = ast.ExceptHandler()
     excpt.type = p[2]
@@ -1485,7 +1564,7 @@ def p_import_stmt1(p):
     ''' import_stmt : import_name '''
     p[0] = p[1]
 
-    
+
 def p_import_stmt2(p):
     ''' import_stmt : import_from '''
     p[0] = p[1]
@@ -1529,7 +1608,7 @@ def p_import_from4(p):
 
 def p_import_from5(p):
     ''' import_from : FROM import_from_dots dotted_name IMPORT import_as_name '''
-    imprt = ast.ImportFrom(module=p[3], names=p[5], level=len(p[2]))
+    imprt = ast.ImportFrom(module=p[3], names=[p[5]], level=len(p[2]))
     imprt.col_offset = 0
     p[0] = imprt
 
@@ -1760,52 +1839,52 @@ def p_comparison_list2(p):
 
 
 def p_comp_op1(p):
-    ''' comp_op : LESS ''' 
+    ''' comp_op : LESS '''
     p[0] = ast.Lt()
 
 
 def p_comp_op2(p):
-    ''' comp_op : GREATER ''' 
+    ''' comp_op : GREATER '''
     p[0] = ast.Gt()
 
 
 def p_comp_op3(p):
-    ''' comp_op : EQEQUAL ''' 
+    ''' comp_op : EQEQUAL '''
     p[0] = ast.Eq()
 
 
 def p_comp_op4(p):
-    ''' comp_op : GREATEREQUAL ''' 
+    ''' comp_op : GREATEREQUAL '''
     p[0] = ast.GtE()
 
 
 def p_comp_op5(p):
-    ''' comp_op : LESSEQUAL ''' 
+    ''' comp_op : LESSEQUAL '''
     p[0] = ast.LtE()
 
 
 def p_comp_op6(p):
-    ''' comp_op : NOTEQUAL ''' 
+    ''' comp_op : NOTEQUAL '''
     p[0] = ast.NotEq()
 
 
 def p_comp_op7(p):
-    ''' comp_op : IN ''' 
+    ''' comp_op : IN '''
     p[0] = ast.In()
 
 
 def p_comp_op8(p):
-    ''' comp_op : NOT IN ''' 
+    ''' comp_op : NOT IN '''
     p[0] = ast.NotIn()
 
 
 def p_comp_op9(p):
-    ''' comp_op : IS ''' 
+    ''' comp_op : IS '''
     p[0] = ast.Is()
 
 
 def p_comp_op10(p):
-    ''' comp_op : IS NOT ''' 
+    ''' comp_op : IS NOT '''
     p[0] = ast.IsNot()
 
 
@@ -1843,7 +1922,7 @@ def p_xor_expr2(p):
     for op, right in p[2]:
         node = ast.BinOp(left=node, op=op, right=right)
     p[0] = node
-    
+
 
 def p_xor_expr_list1(p):
     ''' xor_expr_list : CIRCUMFLEX and_expr '''
@@ -2089,7 +2168,7 @@ def p_atom3(p):
     elif isinstance(info, GeneratorInfo):
         node = ast.GeneratorExp(elt=info.elt, generators=info.generators)
     else:
-        # We have a test node by itself in parenthesis controlling 
+        # We have a test node by itself in parenthesis controlling
         # order of operations, so just return the node.
         node = info
     p[0] = node
@@ -2110,7 +2189,7 @@ def p_atom5(p):
     else:
         raise TypeError('Unexpected node for listmaker: %s' % info)
     p[0] = node
-    
+
 
 def p_atom6(p):
     ''' atom : LBRACE RBRACE '''
@@ -2166,7 +2245,7 @@ def p_atom_string_list2(p):
 
 
 # We dont' allow the backqoute atom from standard Python. Just
-# use repr(...). This simplifies the grammar since we don't have 
+# use repr(...). This simplifies the grammar since we don't have
 # to define a testlist1.
 
 
@@ -2252,8 +2331,8 @@ def p_trailer1(p):
 def p_trailer2(p):
     ''' trailer : LPAR arglist RPAR '''
     args = p[2]
-    p[0] = ast.Call(args=args.args, keywords=args.keywords, 
-                    starargs=args.starargs, kwargs=args.kwargs) 
+    p[0] = ast.Call(args=args.args, keywords=args.keywords,
+                    starargs=args.starargs, kwargs=args.kwargs)
 
 
 def p_trailer3(p):
@@ -2264,7 +2343,7 @@ def p_trailer3(p):
 def p_trailer4(p):
     ''' trailer : DOT NAME '''
     p[0] = ast.Attribute(attr=p[2], ctx=Load)
-    
+
 
 def p_subscriptlist1(p):
     ''' subscriptlist : subscript '''
@@ -2349,7 +2428,7 @@ def p_subscript9(p):
 
 def p_subscript10(p):
     ''' subscript : test COLON test '''
-    p[0] = ast.Slice(lower=p[1], uppper=p[3], step=None)
+    p[0] = ast.Slice(lower=p[1], upper=p[3], step=None)
 
 
 def p_subscript11(p):
@@ -2572,6 +2651,50 @@ def p_arglist10(p):
     p[0] = Arguments(args=args, keywords=kws, kwargs=p[3])
 
 
+def p_arglist11(p):
+    ''' arglist : STAR test COMMA argument '''
+    keyword = p[4]
+    if isinstance(keyword, ast.keyword):
+        p[0] = Arguments(keywords=[keyword], starargs=p[2])
+    else:
+        msg = 'only named arguments may follow *expression'
+        tok = FakeToken(p.lexer.lexer, p.lineno(1))
+        syntax_error(msg, tok)
+
+
+def p_arglist12(p):
+    ''' arglist : STAR test COMMA argument COMMA DOUBLESTAR test '''
+    keyword = p[4]
+    if isinstance(keyword, ast.keyword):
+        p[0] = Arguments(keywords=[keyword], starargs=p[2], kwargs=p[7])
+    else:
+        msg = 'only named arguments may follow *expression'
+        tok = FakeToken(p.lexer.lexer, p.lineno(1))
+        syntax_error(msg, tok)
+
+
+def p_arglist13(p):
+    ''' arglist : STAR test COMMA arglist_list argument '''
+    keywords = p[4] + [p[5]]
+    for kw in keywords:
+        if not isinstance(kw, ast.keyword):
+            msg = 'only named arguments may follow *expression'
+            tok = FakeToken(p.lexer.lexer, p.lineno(1))
+            syntax_error(msg, tok)
+    p[0] = Arguments(keywords=keywords, starargs=p[2])
+
+
+def p_arglist14(p):
+    ''' arglist : STAR test COMMA arglist_list argument COMMA DOUBLESTAR test '''
+    keywords = p[4] + [p[5]]
+    for kw in keywords:
+        if not isinstance(kw, ast.keyword):
+            msg = 'only named arguments may follow *expression'
+            tok = FakeToken(p.lexer.lexer, p.lineno(1))
+            syntax_error(msg, tok)
+    p[0] = Arguments(keywords=keywords, starargs=p[2], kwargs=p[8])
+
+
 def p_arglist_list1(p):
     ''' arglist_list : argument COMMA '''
     p[0] = [p[1]]
@@ -2589,7 +2712,7 @@ def p_argument1(p):
 
 def p_argument2(p):
     ''' argument : test comp_for '''
-    p[0] = ast.GeneratorExp(elt=p[1], generators=p[2]) 
+    p[0] = ast.GeneratorExp(elt=p[1], generators=p[2])
 
 
 # This keyword argument needs to be asserted as a NAME, but using NAME
@@ -3023,17 +3146,17 @@ def p_fplist4(p):
 def p_fplist_list1(p):
     ''' fplist_list : COMMA fpdef '''
     p[0] = [p[2]]
-  
+
 
 def p_fplist_list2(p):
     ''' fplist_list : fplist_list COMMA fpdef '''
     p[0] = p[1] + [p[3]]
 
- 
+
 def p_error(t):
     msg = 'invalid syntax'
     lexer = t.lexer
-    # Ply has a weird thing where sometimes we get the EnamlLexer and 
+    # Ply has a weird thing where sometimes we get the EnamlLexer and
     # other times we get the Ply lexer
     if isinstance(lexer, EnamlLexer):
         lexer = lexer.lexer
@@ -3055,7 +3178,7 @@ _parser = yacc.yacc(
 def parse(enaml_source, filename='Enaml'):
     # All errors in the parsing and lexing rules are raised as a custom
     # ParsingError. This exception object can be called to return the
-    # actual exception instance that should be raised. This is done 
+    # actual exception instance that should be raised. This is done
     # because Ply enters an error recovery mode whenever a SyntaxError
     # is raised from within a rule. We don't want error recovery, we'd
     # rather just fail immediately. So this mechanism allows us to
